"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Server_1 = require("../Server");
const services = require("../services");
const chai_1 = require("chai");
const tmp = require("tmp");
const superagent = require("superagent");
const Realm = require("realm");
const uuid = require("uuid");
const path = require("path");
describe('Test UsernamePasswordAuthService', function () {
    let server;
    let service;
    let tmpDir;
    before(() => __awaiter(this, void 0, void 0, function* () {
        tmpDir = tmp.dirSync();
        Realm.clearTestState();
        server = new Server_1.Server({
            dataPath: tmpDir.name,
            logLevel: 'off'
        });
        server.addService(new services.UsernamePasswordAuthService({
            privateKeyPath: path.join(server.config.dataPath, "keys/auth.key")
        }));
        server.addService(new services.SyncService({
            publicKeyPath: path.join(server.config.dataPath, "keys/auth.pub"),
            dataPath: path.join(server.config.dataPath, "sync"),
        }));
        yield server.start();
        service = server.services.find(s => Reflect.get(s.constructor, 'serviceName') === 'auth');
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        Realm.clearTestState();
        if (server) {
            yield server.shutdown();
        }
    }));
    it.skip('DEPRECATED test user creation', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            const response = yield superagent
                .post('http://localhost:9080/auth/')
                .send({ username: username, password: password, register: true });
            chai_1.expect(response.body.created).to.be.true;
            chai_1.expect(response.body.isAdmin).to.be.false;
            chai_1.expect(response.body.userId).to.eq(username);
            chai_1.expect(response.body.authToken).to.be.a('string');
        });
    });
    it.skip('DEPRECATED test existing user authentication', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            const createResponse = yield superagent
                .post('http://localhost:9080/auth/')
                .send({ username: username, password: password, register: true });
            chai_1.expect(createResponse.body.created).to.be.true;
            chai_1.expect(createResponse.body.isAdmin).to.be.false;
            chai_1.expect(createResponse.body.userId).to.eq(username);
            chai_1.expect(createResponse.body.authToken).to.be.a('string');
            const loginResponse = yield superagent
                .post('http://localhost:9080/auth/')
                .send({ username: username, password: password });
            chai_1.expect(loginResponse.body.created).to.be.false;
            chai_1.expect(loginResponse.body.isAdmin).to.be.false;
            chai_1.expect(loginResponse.body.userId).to.eq(username);
            chai_1.expect(loginResponse.body.authToken).to.be.a('string');
        });
    });
    it.skip('DEPRECATED test existing user bad password', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            const createResponse = yield superagent
                .post('http://localhost:9080/auth/')
                .send({ username: username, password: password });
            chai_1.expect(createResponse.body.created).to.be.true;
            const loginResponse = yield superagent
                .post('http://localhost:9080/auth/')
                .send({ username: username, password: 'bad password' });
            chai_1.expect(loginResponse.error).to.be.true;
        });
    });
    it.skip('DEPRECATED test get realm refresh token', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const username = `Manny${uuid.v4()}`;
            const password = `bat${uuid.v4()}`;
            const createResponse = yield superagent
                .post('http://localhost:9080/auth/')
                .send({ username: username, password: password, register: true, is_admin: true });
            chai_1.expect(createResponse.body.refresh_token.token).to.be.a('string');
            const realmTokenResponse = yield superagent
                .post('http://localhost:9080/auth/')
                .send({ path: '/myrealm', data: createResponse.body.refresh_token.token });
            chai_1.expect(realmTokenResponse.body.access_token.token).to.be.a('string');
        });
    });
    it('should be able to programmatically store and retrieve the same user', function () {
        return __awaiter(this, void 0, void 0, function* () {
            service.createOrUpdateUser('testuserId', true);
            const foundUser = service.getUserById('testuserId');
            chai_1.expect(foundUser).to.not.be.undefined;
            chai_1.expect(foundUser).to.not.be.null;
            chai_1.expect(foundUser.userId).to.eq('testuserId');
            chai_1.expect(foundUser.isAdmin).to.be.true;
        });
    });
    it('should be able to salt and hash a clear text password', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const salt = yield service.createSalt(10000);
            const hash = yield service.hashPassword('goodpassword', salt);
            const passwordIsSame = yield service.comparePassword('goodpassword', salt, hash);
            const passwordIsNotSame = yield service.comparePassword('badpassword', salt, hash);
            chai_1.expect(passwordIsSame).to.be.true;
            chai_1.expect(passwordIsNotSame).to.be.false;
        });
    });
    it('should respond with error if user doesn´t exist', function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield superagent
                .post('http://localhost:9080/auth/')
                .send({ app_id: '', data: 'some-unknown-user', provider: 'password', user_info: {
                    password: 'whatever'
                } }).then(() => {
                chai_1.assert.fail("Expected to fail");
            }).catch((err) => {
                chai_1.expect(err.status).to.equal(400);
                chai_1.expect(err.response.body.code).to.equal(611);
                chai_1.expect(err.response.body.status).to.equal(400);
                chai_1.expect(err.response.body.title).to.equal("The provided credentials are invalid or a user does not exist");
            });
        });
    });
});
//# sourceMappingURL=UsernamePasswordAuthService.spec.js.map